@@ -27,6 +27,39 @@
 # along with IFC. If not, see <http://www.gnu.org/licenses/>.                  #
 ################################################################################
 
+
+#' @title Report Layout Extraction
+#' @description
+#' Extracts report layout from `IFC_graphs` object.
+#' @param graphs an `IFC_graphs` object extracted with features extracted.
+#' @return a list containing:\cr
+#' -lay, a 3 columns (N, x, y) data.frame, where N is the graph index and x and y its coordinates on the layout,\cr
+#' -mat, a matrix describing the layout.
+#' @keywords internal
+layoutReport <- function(graphs) {
+  assert(graphs, cla="IFC_graphs")
+  lay=structure(as.data.frame(matrix(integer(),nrow=0, ncol=3)), names=c("N","x","y"))
+  lay_mat=structure(matrix(integer(),nrow=0,ncol=0),"row.vars"=list(y=character()), "col.vars"=list(x=character()))
+  if(length(graphs) != 0) {
+    lay=lapply(graphs, FUN=function(g) with(g, c(x=xlocation,y=ylocation)))
+    lay=as.data.frame(do.call(rbind, lay))
+    row.names(lay)=seq_along(graphs)
+    lay=by(lay, lay$y, FUN=function(d) {
+      d=d[order(d$x), ]
+      d$x=seq_along(d$x)
+      cbind("N"=as.integer(row.names(d)), d, stringsAsFactors=FALSE)
+    })
+    lay=lapply(seq_along(c(lay)), FUN=function(i) {
+      d=lay[[i]]
+      d$y=i
+      return(d)
+    })
+    lay=do.call("rbind", c(lay, make.row.names=FALSE))
+    lay_mat=ftable(by(lay$N, lay[,c("y","x")], FUN=function(x) x))
+  }
+  return(list(lay=lay, mat=unclass(lay_mat)))
+}
+
 #' @title Report File Creation
 #' @description
 #' Checks if report files can be created.
@@ -114,9 +147,11 @@ tryReportFileCreation <- function(fileName, write_to, overwrite = FALSE) {
 #' @description
 #' Generates graph report (plot + statistics) from `IFC_data` object.
 #' @param obj an `IFC_data` object extracted with features extracted.
-#' @param selection when provided, indices of desired graphs.\cr
-#' In such case 'onepage' parameter is set to FALSE.\cr
-#' Note that indices are read from left to right, from top to bottom. 
+#' @param selection indices of desired graphs. It can be provided as an integer vector or as a matrix.\cr
+#' In such case, the layout of the matrix will reflect the layout of the extracted graphs.\cr
+#' NA value will result in an empty place.\cr
+#' Otherwise, when 'selection' is provided as a vector not identical to seq_along(obj$graphs), 'onepage' parameter will be set to FALSE.\cr
+#' Note that indices are read from left to right, from top to bottom. Default is missing for extracting all graphs.
 #' @param onepage whether to generate a pdf with all graphs on one page or not. Default is TRUE.
 #' @param color_mode Whether to extract colors from obj in white or black mode. Default is "white".
 #' @param add_key whether to draw a "global" key under title or in the first "panel" or "both". Default is "panel".\cr
@@ -126,7 +161,7 @@ tryReportFileCreation <- function(fileName, write_to, overwrite = FALSE) {
 #' -"light", the default, will only display points of same coordinates that are among the other layers.\cr
 #' -"full" will display all the layers.
 #' @param trunc_labels maximum number of characters to display for labels. Default is 38.
-#' @param trans transformation function for density graphs. If missing the default, the BasePop[[1]]$densitytrans, if any, will be retrieved, otherwise asinh will be used.
+#' @param trans name of the transformation function for density graphs. If missing the default, the BasePop[[1]]$densitytrans, if any, will be retrieved, otherwise "asinh" will be used.
 #' @param bin default number of bin used for histogram. Default is missing.
 #' @param viewport Either "ideas", "data" or "max" defining limits used for the graph. Default is "ideas".\cr
 #' -"ideas" will use same limits as the one defined in ideas.\cr
@@ -137,9 +172,9 @@ tryReportFileCreation <- function(fileName, write_to, overwrite = FALSE) {
 #' @return a list with onepage, layout, layout_matrix, graphs, grobs, and stats.
 #' @keywords internal
 CreateGraphReport <- function(obj, selection, onepage=TRUE,
-                            color_mode=c("white","black")[1], add_key="panel", precision=c("light","full")[1],    # parameters to pass to plotGraph
-                            trunc_labels=38, trans=asinh, bin, viewport="ideas",                                  # parameters to pass to plotGraph
-                            display_progress=TRUE, ...) {
+                              color_mode=c("white","black")[1], add_key="panel", precision=c("light","full")[1],    # parameters to pass to plotGraph
+                              trunc_labels=38, trans="asinh", bin, viewport="ideas",                                # parameters to pass to plotGraph
+                              display_progress=TRUE, ...) {
   dots = list(...)
   dv = dev.list()
   on.exit(while(!identical(dv, dev.list())) {
@@ -162,10 +197,22 @@ CreateGraphReport <- function(obj, selection, onepage=TRUE,
   } else {
     bin=na.omit(as.integer(bin)); assert(bin, len=1, typ="integer")
   }
-  if(!"draw" %in% names(dots)) dots$draw = FALSE
-  if(!"stats_print" %in% names(dots)) dots$stats_print = FALSE
+  do_draw = na.omit(dots$draw)
+  if(length(do_draw) != 1 || do_draw != FALSE) {
+    do_draw = TRUE
+  } else {
+    do_draw = FALSE
+  }
+  dots$draw = FALSE
+  do_stats = na.omit(dots$stats_print)
+  if(length(do_stats) != 1 || do_stats != FALSE) {
+    do_stats = TRUE
+  } else {
+    do_stats = FALSE
+  }
+  dots$stats_print = FALSE
   plotGraph_args = c(dots, list(obj=obj, color_mode=color_mode, add_key=add_key,
-                        precision=precision, trunc_labels=trunc_labels, viewport=viewport))
+                     precision=precision, trunc_labels=trunc_labels, viewport=viewport))
   if(length(bin) != 0) plotGraph_args = c(plotGraph_args, list(bin=bin))
   if(!missing(trans)) plotGraph_args = c(plotGraph_args, list(trans=trans))
   title_progress = basename(obj$fileName)
@@ -175,31 +222,27 @@ CreateGraphReport <- function(obj, selection, onepage=TRUE,
   if(length(G)==0) stop("there is no graph defined in 'obj'")
   
   # defines layout
-  lay=lapply(G, FUN=function(g) with(g, c(x=xlocation,y=ylocation)))
-  lay=as.data.frame(do.call(rbind, lay))
-  row.names(lay)=1:length(G)
-  lay=by(lay, lay$y, FUN=function(d) {
-    d=d[order(d$x), ]
-    d$x=seq_along(d$x)
-    cbind("N"=as.integer(row.names(d)), d, stringsAsFactors=FALSE)
-  })
-  lay=lapply(1:length(lay), FUN=function(i) {
-    d=lay[[i]]
-    d$y=i
-    return(d)
-  })
-  lay=do.call("rbind", c(lay, make.row.names=FALSE))
-  lay_mat=ftable(by(lay$N, lay[,c("y","x")], FUN=function(x) x))
-  if(missing(selection)) {
-    selection = 1:length(G)
+  lay=layoutReport(G)
+  lay_mat=lay$mat
+  lay=lay$lay
+  if(missing(selection) || (length(suppressWarnings(na.omit(as.integer(c(selection))))) == 0)) {
+    selection = seq_along(G)
   } else {
-    if(!all(selection%in%(1:length(G)))) stop("'selection' refers to graph absent from 'obj'")
-    if(!identical(selection, 1:length(G))) onepage = FALSE
+    if(!all(na.omit(selection)%in%(seq_along(G)))) stop("'selection' refers to graph absent from 'obj'")
   }
-  if(onepage) { 
-    G=G[order(lay$N)]
+  if(is.matrix(selection)) {
+    G=G[c(selection)]
+    lay_mat = matrix(seq_along(selection), nrow = nrow(selection))
+    lay_mat[is.na(selection)] <- NA
+    lay = data.frame(N = seq_along(selection), x = (1:ncol(selection)), y = (1:nrow(selection)))
+    lay = lay[!is.na(selection),,drop=FALSE]
   } else {
-    G=G[lay$N[selection]]
+    if(!identical(selection, seq_along(G))) onepage = FALSE
+    if(onepage) {
+      G=G[order(lay$N)]
+    } else {
+      G=G[na.omit(selection)]
+    }
   }
   gl = length(G)
   
@@ -208,29 +251,43 @@ CreateGraphReport <- function(obj, selection, onepage=TRUE,
     pb_gr = newPB(session = dots$session, min = 0, max = gl, initial = 0, style = 3)
     on.exit(endPB(pb_gr), add = TRUE)
   }
+  default_stats_1D = matrix(NA, nrow = 1, ncol = 8)
+  colnames(default_stats_1D) = c("count","perc",
+                                 "x-Min.","x-1st Qu.","x-Median","x-Mean","x-3rd Qu.","x-Max.")
+  default_stats_2D = matrix(NA, nrow = 1, ncol = 14)
+  colnames(default_stats_2D) = c("count","perc",
+                                 "x-Min.","x-1st Qu.","x-Median","x-Mean","x-3rd Qu.","x-Max.",
+                                 "y-Min.","y-1st Qu.","y-Median","y-Mean","y-3rd Qu.","y-Max.")
   suppressWarnings({
     stList = list()
-    grList = lapply(1:gl, FUN=function(i) {
+    grList = lapply(seq_along(G), FUN=function(i) {
       if(display_progress) setPB(pb = pb_gr, value = i, title = title_progress, label = "computing graphs and stats")
-      g = try(do.call(what = plotGraph, args = c(plotGraph_args, list(graph = G[[i]]))), silent = TRUE)
-      if(inherits(x = g, what = "try-error")) {
-        foo = arrangeGrob(grid.text(label = paste0("Error: ", attr(x = g, which = "condition")$message), gp=gpar(col="red"), draw = FALSE),
-                          top = textGrob(paste0("\n",G[[i]]$title), gp = gpar(fontsize = 8, font=2, lineheight=0.5)))
-        if(G[[i]]$type=="histogram") {
-          stats = matrix(NA, nrow = 1, ncol = 8)
-          colnames(stats) = c("count","perc",
-                              "x-Min.","x-1st Qu.","x-Median","x-Mean","x-3rd Qu.","x-Max.")
+      stats = default_stats_1D
+      rownames(stats) = paste0("Error: ", G[[i]]$title)
+      if(length(G[[i]]) != 0) {
+        g = try({
+          p = do.call(what = plotGraph, args = c(plotGraph_args, list(graph = G[[i]])))
+          p$plot = plot_lattice(p)
+          p
+        }, silent = TRUE)
+        if(!do_draw || inherits(x = g, what = "try-error")) {
+          foo = arrangeGrob(grid.text(label = paste0(ifelse(do_draw,"Error: ","'draw' is set to FALSE "), attr(x = g, which = "condition")$message), gp=gpar(col="red"), draw = FALSE),
+                            top = textGrob(paste0("\n",G[[i]]$title), gp = gpar(fontsize = 8, font=2, lineheight=0.5)))
         } else {
-          stats = matrix(NA, nrow = 1, ncol = 14)
-          colnames(stats) = c("count","perc",
-                              "x-Min.","x-1st Qu.","x-Median","x-Mean","x-3rd Qu.","x-Max.",
-                              "y-Min.","y-1st Qu.","y-Median","y-Mean","y-3rd Qu.","y-Max.")
+          if(G[[i]]$type=="histogram") {
+            stats = default_stats_1D
+          } else {
+            stats = default_stats_2D
+          }
+          rownames(stats) = paste0("Error: ", G[[i]]$title)
+          foo = grob(p=g$plot, vp = viewport(x=0.5, y=unit(0.5,"npc")), cl = "lattice")
         }
-        rownames(stats) = paste0("Error: ", G[[i]]$title)
       } else {
-        stats = g$stats
-        foo = grob(p=g$plot, vp = viewport(x=0.5, y=unit(0.5,"npc")), cl = "lattice")
+        do_stats = FALSE
+        do_draw = FALSE
+        foo = grob()
       }
+      if(do_stats) try({stats = plot_stats(g)}, silent = TRUE)
       stList <<- c(stList, list(stats))
       stats = stats[,!grepl("Qu", colnames(stats)),drop=FALSE]
       foo_lay = matrix(c(rep(1,9), c(NA,2,NA)), nrow=4, ncol=3, byrow = TRUE)
@@ -240,9 +297,11 @@ CreateGraphReport <- function(obj, selection, onepage=TRUE,
           stats[8, ] = "..."
         }
       }
-      tab = tableGrob(format(stats, scientific=FALSE, digits=5), theme = ttheme_default(base_size=4, base_family = "serif"))
-      tab$vp <- viewport(x=0.5, y=unit(1,"npc") - 0.5*unit(sum(tab$heights, na.rm=TRUE),"npc"))
-      foo = arrangeGrob(foo, tab, layout_matrix = foo_lay, respect = TRUE)
+      if(do_stats) {
+        tab = tableGrob(format(stats, scientific=FALSE, digits=5), theme = ttheme_default(base_size=4, base_family = "serif"))
+        tab$vp <- viewport(x=0.5, y=unit(1,"npc") - 0.5*unit(sum(tab$heights, na.rm=TRUE),"npc"))
+        foo = arrangeGrob(foo, tab, layout_matrix = foo_lay, respect = TRUE)
+      }
       return(foo)
     })
   })
@@ -253,9 +312,11 @@ CreateGraphReport <- function(obj, selection, onepage=TRUE,
 #' @description
 #' Generates report from `IFC_data` object.
 #' @param obj an `IFC_data` object extracted with features extracted.
-#' @param selection when provided, indices of desired graphs.\cr
-#' In such case 'onepage' parameter is set to FALSE.\cr
-#' Note that indices are read from left to right, from top to bottom. 
+#' @param selection indices of desired graphs. It can be provided as an integer vector or as a matrix.\cr
+#' In such case, the layout of the matrix will reflect the layout of the extracted graphs.\cr
+#' NA value will result in an empty place.\cr
+#' Otherwise, when 'selection' is provided as a vector not identical to seq_along(obj$graphs), 'onepage' parameter will be set to FALSE.\cr
+#' Note that indices are read from left to right, from top to bottom. Default is missing for extracting all graphs.
 #' @param write_to pattern used to export file(s).
 #' Placeholders, like c("\%d/\%s_fromR.pdf", "\%d/\%s_fromR.csv"), will be substituted:\cr
 #' -\%d: with full path directory of 'obj$fileName'\cr
@@ -274,7 +335,7 @@ CreateGraphReport <- function(obj, selection, onepage=TRUE,
 #' -"light", the default, will only display points of same coordinates that are amoung the other layers.\cr
 #' -"full" will display all the layers.
 #' @param trunc_labels maximum number of characters to display for labels. Default is 38.
-#' @param trans transformation function for density graphs. If missing the default, the BasePop[[1]]$densitytrans, if any, will be retrieved, otherwise asinh will be used.
+#' @param trans name of the transformation function for density graphs. If missing the default, the BasePop[[1]]$densitytrans, if any, will be retrieved, otherwise "asinh" will be used.
 #' @param bin default number of bin used for histogram. Default is missing.
 #' @param viewport Either "ideas", "data" or "max" defining limits used for the graph. Default is "ideas".\cr
 #' -"ideas" will use same limits as the one defined in ideas.\cr
@@ -309,7 +370,7 @@ CreateGraphReport <- function(obj, selection, onepage=TRUE,
 #' @export
 ExportToReport = function(obj, selection, write_to, overwrite=FALSE, onepage=TRUE,
                           color_mode=c("white","black")[1], add_key="panel", precision=c("light","full")[1],    # parameters to pass to plotGraph
-                          trunc_labels=38, trans=asinh, bin, viewport="ideas",                                  # parameters to pass to plotGraph
+                          trunc_labels=38, trans="asinh", bin, viewport="ideas",                                  # parameters to pass to plotGraph
                           display_progress=TRUE, ...) {
   dots = list(...)
   dv = dev.list()
@@ -343,11 +404,13 @@ ExportToReport = function(obj, selection, write_to, overwrite=FALSE, onepage=TRU
                             display_progress=display_progress, ...)
     gl = length(foo$graphs)
     if(create_csv) {
-      lapply(1:gl, FUN = function(i) {
+      lapply(seq_along(foo$graphs), FUN = function(i) {
         g = foo$graphs[[i]]
-        write.table(x=rbind(c(g$f1,"x")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
-        if(g$type!="histogram") write.table(x=rbind(c(g$f2,"y")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
-        write.table(x=foo$stats[[i]], file=export_to_csv, append=TRUE, sep=",", col.names = FALSE)
+        if(length(g)!=0) {
+          write.table(x=rbind(c(g$f1,"x")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
+          if(g$type!="histogram") write.table(x=rbind(c(g$f2,"y")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
+          write.table(x=foo$stats[[i]], file=export_to_csv, append=TRUE, sep=",", col.names = FALSE)
+        }
       })
     }
     if(create_pdf) {
@@ -421,7 +484,7 @@ DisplayReport = function(obj, display_progress = TRUE, ...) {
     # TODO add a progress bar
     grid.arrange(grobs = foo$grobs[foo$layout$N], top = title_progress, newpage = TRUE, layout_matrix = foo$matrix, as.table = FALSE)
   } else {
-    for(i in 1:gl) {
+    for(i in seq_along(foo$grobs)) {
       grid.arrange(foo$grobs[[i]], top = title_progress, newpage = TRUE)
       if(display_progress) setPB(pb_pdf, value = i, title = title_progress, label = "plotting each graph")
     }
@@ -431,25 +494,25 @@ DisplayReport = function(obj, display_progress = TRUE, ...) {
 #' @title Batch Generation of Graphical and Statistic Report
 #' @description
 #' Batch creates graphical an statistical report.
-#' @param fileName path to file(s).
-#' @param selection indices of desired graphs. It can be provided as an integer vector or as a matrix.
-#' In such case, the layout of the matrix will reflect the layout of the extracted graphs for each single 'fileName'.
-#' NA value will result in an empty place.
+#' @param fileName,obj either one or the other. Path to file(s) to read from for 'fileName' or list of `IFC_data` objects for obj.
+#' @param selection indices of desired graphs. It can be provided as an integer vector or as a matrix.\cr
+#' In such case, the layout of the matrix will reflect the layout of the extracted graphs for each 'fileName' or ''obj'.\cr
+#' NA value will result in an empty place. When missing, it will be determined by the whole layout of 1st 'fileName' or 'obj' with 'gating' applied when provided
 #' @param write_to pattern used to export file(s).
 #' Placeholders, like c("\%d/\%s_fromR.pdf", "\%d/\%s_fromR.csv"), will be substituted:\cr
-#' -\%d: with full path directory of first element of 'fileName'\cr
-#' -\%p: with first parent directory of first element of 'fileName'\cr
-#' -\%e: with extension of first element of 'fileName' (without leading .)\cr
-#' -\%s: with shortname from first element of 'fileName' (i.e. basename without extension).\cr
-#' Exported file(s) extension(s) will be deduced from this pattern. Note that has to be a .pdf and/or .csv.
+#' -\%d: with full path directory\cr
+#' -\%p: with first parent directory\cr
+#' -\%e: with extension (without leading .)\cr
+#' -\%s: with shortname (i.e. basename without extension).\cr
+#' Exported file(s) extension(s) will be deduced from this pattern using either 1st 'fileName' or 'obj'. Note that has to be a .pdf and/or .csv.
 #' @param overwrite whether to overwrite file or not. Default is FALSE.
 #' Note that if TRUE, it will overwrite file. In addition a warning message will be sent.
 #' @param gating an `IFC_gating` object as extracted by readGatingStrategy(). Default is missing.
-#' If not missing, each file provided in 'fileName' will be extracted by readIFC() and 'gating' will be applied by applyGatingStrategy() to the returned object before creating the report
+#' If not missing, each `IFC_data` provided in 'obj' or read from 'fileName' will be passed to applyGatingStrategy() before creating the report.
 #' @param main the main title of the document. Default is missing.
-#' @param byrow whether to add selected graphs for each file by row or not. Deafult is FALSE.
+#' @param byrow whether to add selected graphs for each file by row or not. Default is FALSE.
 #' @param times number of files to add before starting a new row or column (depending on 'byrow').
-#' @param color_mode Whether to extract colors from obj in white or black mode. Default is "white".
+#' @param color_mode Whether to extract colors in white or black mode. Default is "white".
 #' @param add_key whether to draw a "global" key under title or in the first "panel" or "both". Default is "panel".\cr
 #' Accepted values are either: FALSE, "panel", "global", "both" or c("panel", "global").\cr
 #' Note that it only applies when display is seen as overlaying populations.
@@ -457,7 +520,7 @@ DisplayReport = function(obj, display_progress = TRUE, ...) {
 #' -"light", the default, will only display points of same coordinates that are among the other layers.\cr
 #' -"full" will display all the layers.
 #' @param trunc_labels maximum number of characters to display for labels. Default is 38.
-#' @param trans transformation function for density graphs. If missing the default, the BasePop[[1]]$densitytrans, if any, will be retrieved, otherwise asinh will be used.
+#' @param trans name of the transformation function for density graphs. If missing the default, the BasePop[[1]]$densitytrans, if any, will be retrieved, otherwise "asinh" will be used.
 #' @param bin default number of bin used for histogram. Default is missing.
 #' @param viewport Either "ideas", "data" or "max" defining limits used for the graph. Default is "ideas".\cr
 #' -"ideas" will use same limits as the one defined in ideas.\cr
@@ -466,11 +529,11 @@ DisplayReport = function(obj, display_progress = TRUE, ...) {
 #' @param display_progress whether to display a progress bar. Default is TRUE.
 #' @param ... other parameters to be passed.
 #' @return It invisibly returns full path of exported .pdf and/or .csv file(s).
-#' @keywords internal
-BatchReport <- function(fileName, selection, write_to, overwrite=FALSE, 
+#' @export
+BatchReport <- function(fileName, obj, selection, write_to, overwrite=FALSE, 
                         gating, main, byrow = FALSE, times = 5, 
                         color_mode=c("white","black")[1], add_key="panel", precision=c("light","full")[1],    # parameters to pass to plotGraph
-                        trunc_labels=38, trans=asinh, bin, viewport="ideas",                                  # parameters to pass to plotGraph
+                        trunc_labels=38, trans="asinh", bin, viewport="ideas",                                # parameters to pass to plotGraph
                         display_progress=TRUE, ...) {
   dots = list(...)
   create_pdf = FALSE
@@ -481,6 +544,22 @@ BatchReport <- function(fileName, selection, write_to, overwrite=FALSE,
   on.exit(suppressWarnings(Sys.setlocale("LC_ALL", locale = locale_back)), add = TRUE)
   suppressWarnings(Sys.setlocale("LC_ALL", locale = "English"))
   
+  # check fileName or obj
+  if(missing(obj) && missing(fileName)) stop("you should provide either 'fileName' or 'obj'")
+  if(!missing(fileName) && !missing(obj)) stop("you should provide either 'fileName' or 'obj' not both")
+  if(!missing(fileName) && is.list(fileName) && inherits(fileName[[1]], what="IFC_data")) {
+    warning("'fileName' will be treated as a list of `IFC_data` objects")
+    obj = fileName
+  }
+  is_obj = FALSE
+  if(!missing(obj)) {
+    is_obj = TRUE
+    fileName = sapply(seq_along(obj), FUN = function(i_obj) {
+      assert(obj[[i_obj]], cla="IFC_data")
+      obj[[i_obj]]$fileName
+    })
+  }
+  
   # check file(s) can be created
   can_write = tryReportFileCreation(fileName[1], write_to, overwrite)
   export_to_csv = can_write$csv
@@ -490,11 +569,12 @@ BatchReport <- function(fileName, selection, write_to, overwrite=FALSE,
   create_pdf = length(export_to_pdf) != 0
   write_to = c(export_to_pdf, export_to_csv)
   
-  # checks
+  # checks & coercion
   display_progress = as.logical(display_progress); assert(display_progress, len=1, alw=c(TRUE,FALSE))
   byrow = as.logical(byrow); assert(byrow, len=1, alw=c(TRUE,FALSE))
   times = na.omit(as.integer(times)); assert(times, len = 1)
-  if(missing(selection)) stop("'selection' can't be missing")
+  apply_gating = FALSE
+  if(!missing(gating)) apply_gating = TRUE
   if(missing(bin)) {
     bin = NULL
   } else {
@@ -503,25 +583,38 @@ BatchReport <- function(fileName, selection, write_to, overwrite=FALSE,
   if(!"draw" %in% names(dots)) dots$draw = FALSE
   if(!"stats_print" %in% names(dots)) dots$stats_print = FALSE
   plotGraph_args=c(dots, list(color_mode=color_mode, add_key=add_key,
-                      precision=precision, trunc_labels=trunc_labels,
-                      viewport=viewport))
+                              precision=precision, trunc_labels=trunc_labels,
+                              viewport=viewport))
   if(length(bin) != 0) plotGraph_args = c(plotGraph_args, list(bin=bin))
   if(!missing(trans)) plotGraph_args = c(plotGraph_args, list(trans=trans))
-
+  
+  # layout
   n_tiles = length(fileName)
   s_tiles = seq(from = 1, to = n_tiles, by = 1)
   lay = split(seq(from = 1, to = n_tiles, by = 1), ceiling(seq(from = 1, to = n_tiles, by = 1) / times) )
   if(length(lay) > 1) lay[[length(lay)]] = c(lay[[length(lay)]], rep(NA, times - length(lay[[length(lay)]])))
   lay = do.call(what = ifelse(byrow, "cbind", "rbind"), args = lay)
-  L = length(selection)
-  if(byrow) { LX = 1; LY = L } else { LX = L; LY = 1 }
-  if(inherits(x = selection, what = "matrix")) {
-    LX = ncol(selection)
-    LY = ncol(selection)
+  if(missing(selection)) {
+    if(apply_gating) {
+      selection=gating$graphs
+    } else {
+      if(is_obj) {
+        selection=obj[[1]]$graphs
+      } else {
+        selection=readIFC(fileName=fileName[1], display_progress=FALSE,
+                          extract_features=TRUE, extract_images=FALSE,
+                          extract_offsets=FALSE, extract_stats=FALSE)$graphs
+      }
+    }
+    selection=layoutReport(selection)$mat
   }
-  apply_gating = FALSE
-  if(!missing(gating)) apply_gating = TRUE
-
+  if(!is.matrix(selection)) {
+    selection = na.omit(selection)
+    selection = matrix(selection, nrow=length(selection))
+    if(!byrow) selection = t(selection)
+  }
+  if(length(selection) == 0) selection=matrix(1,ncol=1,nrow=1)
+  
   # backup last state of graphic device
   dv = dev.list()
   if(display_progress) {
@@ -529,41 +622,55 @@ BatchReport <- function(fileName, selection, write_to, overwrite=FALSE,
     on.exit(endPB(pb), add = TRUE)
   }
   tryCatch({
-    grobs = lapply(1:length(fileName), FUN = function(i_file) {
+    grobs = lapply(seq_along(fileName), FUN = function(i_file) {
       if(display_progress) setPB(pb = pb, value = i_file, title = "creating batch report", label = paste0("extracting ",basename(fileName[i_file])))
-      obj = readIFC(fileName=fileName[i_file], display_progress=FALSE, ...)
-      if(apply_gating) obj = applyGatingStrategy(obj=obj, gating=gating, display_progress=FALSE)
-      G = obj$graphs[c(selection)]
-      plotGraph_args = c(plotGraph_args, list(obj=obj))
-      file_lay = matrix(1:L, ncol = LX, nrow = LY)
-      foo = CreateGraphReport(obj, selection, onepage=FALSE,
-                              color_mode=color_mode, add_key=add_key, precision=precision,
-                              trunc_labels=trunc_labels, trans=trans, bin=bin, viewport=viewport,
-                              display_progress=display_progress)
-      gl = length(foo$graphs)
-      if(create_csv) {
-        write.table(x=basename(fileName[i_file]), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
-        lapply(1:gl, FUN = function(i) {
-          g = foo$graphs[[i]]
-          write.table(x=rbind(c(g$f1,"x")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
-          if(g$type!="histogram") write.table(x=rbind(c(g$f2,"y")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
-          write.table(x=foo$stats[[i]], file=export_to_csv, append=TRUE, sep=",", col.names = FALSE)
-        })
-      }
-      file_grob = foo$grobs
-      bar = grobTree(arrangeGrob(grobs=file_grob, ncol=LX, nrow=LY,
-                                 top=textGrob(paste0("\n\n\n",basename(fileName[i_file]),"\n"), 
-                                              gp=gpar(fontsize=14, font=2, col="skyblue4", lineheight=0.5))),
-                     rectGrob(.5,.5,width=unit(.99,"npc"), height=unit(0.99,"npc"),
-                              gp=gpar(lwd=2, col="skyblue4", fill=NA)))
+      tryCatch({
+        if(is_obj) {
+          i_obj = obj[[i_file]]
+        } else {
+          i_obj = readIFC(fileName=fileName[i_file], display_progress=FALSE,
+                          extract_features=TRUE, extract_images=FALSE,
+                          extract_offsets=FALSE, extract_stats=FALSE)
+        }
+        if(apply_gating) i_obj = applyGatingStrategy(obj=i_obj, gating=gating, display_progress=FALSE)
+        plotGraph_args = c(plotGraph_args, list(obj=i_obj))
+        foo = CreateGraphReport(i_obj, selection, onepage=TRUE,
+                                color_mode=color_mode, add_key=add_key, precision=precision,
+                                trunc_labels=trunc_labels, trans=trans, bin=bin, viewport=viewport,
+                                display_progress=display_progress)
+        if(create_csv) {
+          write.table(x=basename(fileName[i_file]), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
+          lapply(seq_along(foo$graphs), FUN = function(i) {
+            g = foo$graphs[[i]]
+            if(length(g)!=0) {
+              write.table(x=rbind(c(g$f1,"x")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
+              if(g$type!="histogram") write.table(x=rbind(c(g$f2,"y")), file=export_to_csv, append=TRUE, sep=",", col.names = FALSE, row.names = FALSE)
+              write.table(x=foo$stats[[i]], file=export_to_csv, append=TRUE, sep=",", col.names = FALSE)
+            }
+          })
+        }
+        grobTree(
+          arrangeGrob(grobs = foo$grobs[foo$layout$N], 
+                       top=textGrob(paste0("\n\n\n",basename(fileName[i_file]),"\n"), 
+                                    gp=gpar(fontsize=14, font=2, col="skyblue4", lineheight=0.5)),
+                       newpage = FALSE,
+                       layout_matrix = foo$matrix, as.table = FALSE),
+          rectGrob(.5,.5,width=unit(.99,"npc"), height=unit(0.99,"npc"), gp=gpar(lwd=2, col="skyblue4", fill=NA)))
+      }, error = function(e) {
+        grobTree(arrangeGrob(grid.text(label = e$message, gp=gpar(col="red"), draw = FALSE),
+                             top=textGrob(paste0("\n\n\n",basename(fileName[i_file]),"\n"), 
+                                          gp=gpar(fontsize=14, font=2, col="skyblue4", lineheight=0.5))),
+                 rectGrob(.5,.5,width=unit(.99,"npc"), height=unit(0.99,"npc"), gp=gpar(lwd=2, col="skyblue4", fill=NA)))
+      })
     })
     if(create_pdf) {
-      pdf(file=write_to, height=3*nrow(lay)*LY*2.54, width=3*ncol(lay)*LX*2.54,
+      pdf(file=write_to, height=3*nrow(lay)*nrow(selection)*2.54, width=3*ncol(lay)*ncol(selection)*2.54,
           family = "serif",
           onefile=TRUE, pagecentre=TRUE, useDingbats=FALSE)
       if(!missing(main)) args=c(args, top=main)
       # no reason to have newpage = TRUE unless pdf is not open
-      args = list(newpage=names(dev.cur()) != "pdf", layout_matrix=lay, as.table=FALSE)
+      args = list(newpage=names(dev.cur()) != "pdf",
+                  layout_matrix=lay, as.table=FALSE)
       tryCatch(do.call(what=grid.arrange, args=c(list(grobs=grobs), args)), 
                error = function(e) { stop(e$message, call.=FALSE) },
                finally = dev.off())
