@@ -91,7 +91,7 @@ ExtractFromDAF <- function(fileName, extract_features = TRUE, extract_images = T
   assert(pnt_in_poly_epsilon, len = 1, typ = "numeric")
   display_progress = as.logical(display_progress); assert(display_progress, len = 1, alw = c(TRUE, FALSE))
   fileName = normalizePath(fileName, winslash = "/", mustWork = FALSE)
-  toskip=cpp_scanFirst(fname = fileName, target = "</Assay>", start = 0, end = 0)
+  toskip=cpp_scanFirst(fileName, charToRaw('</Assay>'), start = 0, end = 0)
   if(toskip == 0) stop(paste0(fileName, "\ndoes not seem to be well formatted: </Assay> not found")) 
   toskip = toskip + nchar("</Assay>") - 1
   tmp=read_xml(readBin(con = fileName, what = "raw", n = toskip), options=c("HUGE","RECOVER","NOENT","NOBLANKS","NSCLEAN"))
@@ -417,8 +417,8 @@ ExtractFromDAF <- function(fileName, extract_features = TRUE, extract_images = T
       })
       plots=mapply(plots, plots_tmp, FUN = append, SIMPLIFY = FALSE)
       plots_tmp=c("xlocation","ylocation","scaletype","xmin","xmax","ymin","ymax","axislabelsfontsize","axistickmarklabelsfontsize",
-                  "graphtitlefontsize","regionlabelsfontsize","bincount","histogramsmoothingfactor","xsize","ysize","splitterdistance")
-      plots=lapply(plots, FUN=function(x) {replace(x, plots_tmp, lapply(x[plots_tmp], as.numeric))})
+                  "graphtitlefontsize","regionlabelsfontsize","bincount","histogramsmoothingfactor","xsize","ysize","splitterdistance","maxpoints")
+      plots=lapply(plots, FUN=function(x) {plots_tmp = plots_tmp[plots_tmp %in% names(x)];replace(x, plots_tmp, lapply(x[plots_tmp], as.numeric))})
       plot_order=sapply(plots, FUN=function(i_plot) as.numeric(i_plot[c("xlocation", "ylocation")]))
       plots=plots[order(unlist(plot_order[1,]),unlist(plot_order[2,]))]
       rm(list=c("plots_tmp", "plot_order"))
