@@ -57,9 +57,9 @@
 #' @param verbosity quantity of information displayed when verbose is TRUE; 1: normal, 2: rich. Default is 1.
 #' @param overwrite whether to overwrite file or not. Default is FALSE.\cr
 #' Note that if TRUE, it will overwrite exported file if path of 'fileName' and deduced from 'write_to' arguments are different.
-#' Otherwise, you will get an error saying that overwritting source file is not allowed.\cr
+#' Otherwise, you will get an error saying that overwriting source file is not allowed.\cr
 #' Note also that an original file, i.e. generated by IDEAS(R) or INSPIRE(R), will never be overwritten.
-#' Otherwise, you will get an error saying that overwritting original file is not allowed.\cr
+#' Otherwise, you will get an error saying that overwriting original file is not allowed.\cr
 #' @param display_progress whether to display a progress bar. Default is TRUE.
 #' @param ... other arguments to be passed.
 #' @details when 'extract_features' is set TRUE, only features stored in binary format will be extracted if found.\cr
@@ -116,7 +116,7 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
     if(!overwrite) stop(paste0("file ", write_to, " already exists"))
     if(tolower(fileName) == tolower(write_to)) stop("you are trying to overwrite source file which is not allowed")
     tryCatch({
-      IFD_export = getIFD(fileName = write_to, offsets = "first", force_trunc = FALSE, trunc_bytes = 8, verbose = FALSE, bypass = FALSE)[[1]] 
+      IFD_export = getIFD(fileName = write_to, offsets = "first", force_trunc = FALSE, trunc_bytes = 8, verbose = FALSE, bypass = TRUE)[[1]]
     }, error = function(e) {
       stop(paste0(write_to, "\ndoes not seem to be well formatted:\n", e$message), call. = FALSE)
     })
@@ -125,10 +125,9 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
     overwritten = TRUE
   }
   file_w = ifelse(overwritten, tmp_file, write_to)
-
+  
   # Extract important values
-  IFDs = getIFD(fileName = fileName, offsets = "first", force_trunc = FALSE, trunc_bytes = 8, verbose = FALSE, bypass = FALSE)
-  nobj = IFDs[[1]]$tags[["33018"]]$map
+  IFDs = getIFD(fileName = fileName, offsets = "first", force_trunc = FALSE, trunc_bytes = 8, verbose = FALSE, bypass = TRUE)
   is_binary = as.logical(IFDs[[1]]$tags[["33082"]]$map)
   if(length(is_binary) == 0) is_binary = FALSE
   V = NULL
@@ -146,6 +145,27 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
   
   if(length(V) > 1) Files = normalizePath(paste(dirname(fileName), basename(V), sep = "/"), winslash = "/", mustWork = FALSE)
   
+  compute_offsets = TRUE
+  if(!missing(offsets)) {
+    if(!("IFC_offset" %in% class(offsets))) {
+      warning("provided offsets do not match with expected ones, offsets will be recomputed", immediate. = TRUE, call. = FALSE)
+    } else {
+      if(attr(offsets, "checksum") != checksumXIF(fileName)) {
+        warning("provided offsets do not match with expected ones, offsets will be recomputed", immediate. = TRUE, call. = FALSE)
+      } else {
+        compute_offsets = FALSE
+      }
+    }
+  }
+  if(compute_offsets) {
+    offsets = suppressMessages(getOffsets(fileName = fileName, fast = fast, display_progress = display_progress)) 
+  } else {
+    fast = TRUE
+  }
+  XIF_test = ifelse(length(attr(offsets, "test")) == 0, testXIF(fileName), attr(offsets, "test"))
+  XIF_step = as.integer(XIF_test == 1) + 1L
+  nobj = as.integer(attr(x = offsets, which = "obj_count"))
+  
   if(missing(objects)) {
     message("\nAll objects will be extracted\n")
     objects = as.integer(0:(nobj - 1))
@@ -165,24 +185,6 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
   }
   objects = objects[order(objects)] # since it appears that objects are stored with increasing number
   
-  compute_offsets = TRUE
-  if(!missing(offsets)) {
-    if(!("IFC_offset" %in% class(offsets))) {
-      warning("provided offsets do not match with expected ones, offsets will be recomputed", immediate. = TRUE, call. = FALSE)
-    } else {
-      if(attr(offsets, "checksum") != checksumXIF(fileName)) {
-        warning("provided offsets do not match with expected ones, offsets will be recomputed", immediate. = TRUE, call. = FALSE)
-      } else {
-        compute_offsets = FALSE
-      }
-    }
-  }
-  if(compute_offsets) {
-    offsets = suppressMessages(getOffsets(fileName = fileName, fast = fast, display_progress = display_progress))
-  } else {
-    fast = TRUE
-  }
-  
   # Initialize values
   obj_id = - 1
   L = length(objects)
@@ -190,7 +192,6 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
   cur_env = environment()
   offsets = subsetOffsets(offsets = offsets, objects = objects, image_type = c("img", "msk"))
   offsets = c(IFDs[[1]]$curr_IFD_offset, offsets)
-  
   off_number = length(offsets)
   
   # unwanted tags
@@ -203,9 +204,12 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
   # 33083 corresponds to Features values in merged or subset
   # 33090, 33091, 33092, 33093, 33094 corresponds to tags we add to track objects origin
   unwanted = c(33004, 33005, 33018, 33080, 33081, 33082, 33083, 33090, 33091, 33092, 33093, 33094)
+  # TODO ask amnis if we can remove unnecessary tags for masks ?
+  # i.e. only keep: 254, 256, 257, 258, 259, 262, 273, 277, 278, 279, 306, 33002, 33016, 33017
   
   # tags of StripOffsets (273) and TileOffsets (324)
-  off_tags = c(273, 324)
+  # off_tags = c(273, 324)
+  off_tags = 273
   
   # open connections for reading and writing
   toread = file(description = fileName, open = "rb")
@@ -230,103 +234,80 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
     }
     OBJECT_ID = NULL
     for(i_off in 1:off_number) {
+      extra = list()
       # extract IFD
-      IFD = cpp_getTAGS(fname = fileName, offset = offsets[i_off], trunc_bytes = 8, force_trunc = FALSE, verbose = VER)
-      cur_obj = IFD$infos
-      if(cur_obj$TYPE == 2) OBJECT_ID = cur_obj$OBJECT_ID
+      IFD = cpp_fastTAGS(fname = fileName, offset = offsets[i_off], verbose = VER)
+      ifd = IFD$tags[!(names(IFD$tags) %in% unwanted)]
+      TYPE = IFD$tags[["33002"]]$val
+      if(any(TYPE == 2)) OBJECT_ID = IFD$tags[["33003"]]$val
+      # if((length(TYPE) != 0) && (TYPE == 2)) OBJECT_ID = OBJECT_ID
       if(length(OBJECT_ID) == 1) {
         if(fast) {
           expected_obj = as.integer(gsub("^.*_(.*)$", "\\1", names(offsets[i_off])))
-          if(cur_obj$TYPE == 2) if(OBJECT_ID != expected_obj) {
+          if(TYPE == 2) if(OBJECT_ID != expected_obj) {
             warning("Extracted object_id [",OBJECT_ID,"] differs from expected one [",expected_obj,"]", call. = FALSE, immediate. = TRUE)
           }
         }
       }
-      if(display_progress) {
-        setPB(pb1, value = (obj_id+1)/2, title = title_progress, label = "objects subsetting")
-      }
-      
-      # go to file IFD offset
-      seek(toread, offsets[i_off])
-      
-      # read number of directory entries
-      readBin(toread, what = "raw", n = 2, endian = r_endian)
-      # TODO ask amnis if we can remove unecessary tags for masks ?
-      # i.e. only keep: 254, 256, 257, 258, 259, 262, 273, 277, 278, 279, 306, 33002, 33016, 33017
-      # IFD$tags = IFD$tags[c(254, 256, 257, 258, 259, 262, 273, 277, 278, 279, 306, 33002, 33016, 33017)]
-      ifd = sapply(IFD$tags, simplify = FALSE, FUN=function(i_tag) {
-        add_content = raw()
-        # read entry
-        min_content = readBin(toread, what = "raw", n = 12, endian = r_endian)
-        # save current position in read
-        r_pos = seek(toread)
-        # remove unwanted tags
-        if(i_tag$tag %in% unwanted) return(list(min_content = raw(), add_content = raw()))
-        # extract extra content when value is an offset
-        if(i_tag$off || (i_tag$tag %in% off_tags)) {
-          # go to this offset in read
-          seek(toread, i_tag$val)
-          # extra content
-          if(i_tag$tag %in% off_tags) {
-            if(i_tag$tag == 273) add_content = readBin(toread, what = "raw", n = IFD$tags[["279"]]$map, endian = r_endian)
-            if(i_tag$tag == 324) add_content = readBin(toread, what = "raw", n = IFD$tags[["325"]]$map, endian = r_endian)
-          } else {
-            add_content = readBin(toread, what = "raw", n = i_tag$byt, endian = r_endian)
-          }
-          # go to former position in read
-          seek(toread, r_pos)
-        }
-        return(list(min_content = min_content, add_content = add_content))
-      })
-      # remove unwanted tags
-      ifd = ifd[sapply(ifd, FUN=function(i_tag) length(i_tag$min_content)!=0)]
+      if(display_progress) setPB(pb1, value = (obj_id+1)/XIF_step, title = title_progress, label = "objects subsetting")
       
       # extract features values in 1st IFD
-      if(IFD$infos$TYPE == 1) {
+      if(i_off == 1) {
         features = list()
-        if(extract_features) {
-          tryCatch({
-            obj_shift = 0
-            for(f in Files) {
-              if(!file.exists(f)) {
-                features = list()
-                stop(f, "\nCan't find file")
-              }
-              toread2 = file(f, open = "rb")
-              tryCatch({
-                IFD_f = getIFD(fileName = f, offsets = "first", trunc_bytes = 8, force_trunc = FALSE, verbose = FALSE, bypass = TRUE)[[1]]
-                is_binary = as.logical(IFD_f$tags[["33082"]]$map)
-                if(length(is_binary) == 0) is_binary = FALSE
-                if(!is_binary) {
+        if(XIF_test >= 0) {
+          if(extract_features) {
+            tryCatch({
+              obj_shift = 0
+              for(f in Files) {
+                if(!file.exists(f)) {
                   features = list()
-                  stop(f, "\nCan't deal with non-binary features")
+                  stop(f, "\nCan't find file")
                 }
-
-                feat_where = ifelse((length(V) == 0) && (length(IFD_f$tags[["33080"]]$map) != 0) && (IFD_f$tags[["33080"]]$val < file.size(fileName)),
-                                    ifelse(length(IFD_f$tags[["33080"]]$map)==0, 
-                                           stop("can't find pointer '33080' to extract features"), 
-                                           IFD_f$tags[["33080"]]$val),
-                                    ifelse(length(IFD_f$tags[["33083"]]$map)==0, 
-                                           stop("can't find pointer '33083' to extract features"), 
-                                           IFD_f$tags[["33083"]]$val))
-                seek(toread2, feat_where)
-                
-                obj_number_r = readBin(toread2, what = "raw", n = 4, endian = r_endian)
-                obj_number = readBin(obj_number_r, what = "double", size = 4, n = 1, endian = r_endian)
-                feat_number_r = readBin(toread2, what = "raw", n = 4, endian = r_endian)
-                feat_number = readBin(feat_number_r, what = "double", size = 4, n = 1, endian = r_endian)
-                
-                if((length(obj_number) == 0) || (length(feat_number) == 0)) {
-                  features = list()
-                  stop(f, "\nBinary features is of length 0")
-                }
-                
-                title_progress = basename(f)
-                if(display_progress) {
-                  pb3 = newPB(session = dots$session, title = title_progress, label = "extracting features information", min = 0, max = obj_number, initial = 0, style = 3)
-                  tryCatch({
+                toread2 = file(f, open = "rb")
+                tryCatch({
+                  IFD_f = getIFD(fileName = f, offsets = "first", trunc_bytes = 8, force_trunc = FALSE, verbose = FALSE, bypass = TRUE)[[1]]
+                  is_binary = as.logical(IFD_f$tags[["33082"]]$map)
+                  if(length(is_binary) == 0) is_binary = FALSE
+                  if(!is_binary) {
+                    features = list()
+                    stop(f, "\nCan't deal with non-binary features")
+                  }
+                  
+                  feat_where = ifelse((length(V) == 0) && (length(IFD_f$tags[["33080"]]$map) != 0) && (IFD_f$tags[["33080"]]$val < file.size(fileName)),
+                                      ifelse(length(IFD_f$tags[["33080"]]$map)==0, 
+                                             stop("can't find pointer '33080' to extract features"), 
+                                             IFD_f$tags[["33080"]]$val),
+                                      ifelse(length(IFD_f$tags[["33083"]]$map)==0, 
+                                             stop("can't find pointer '33083' to extract features"), 
+                                             IFD_f$tags[["33083"]]$val))
+                  seek(toread2, feat_where)
+                  
+                  obj_number_r = readBin(toread2, what = "raw", n = 4, endian = r_endian)
+                  obj_number = readBin(obj_number_r, what = "double", size = 4, n = 1, endian = r_endian)
+                  feat_number_r = readBin(toread2, what = "raw", n = 4, endian = r_endian)
+                  feat_number = readBin(feat_number_r, what = "double", size = 4, n = 1, endian = r_endian)
+                  
+                  if((length(obj_number) == 0) || (length(feat_number) == 0)) {
+                    features = list()
+                    stop(f, "\nBinary features is of length 0")
+                  }
+                  
+                  title_progress = basename(f)
+                  if(display_progress) {
+                    pb3 = newPB(session = dots$session, title = title_progress, label = "extracting features information", min = 0, max = obj_number, initial = 0, style = 3)
+                    tryCatch({
+                      features = c(features, lapply(1:obj_number, FUN = function(i_obj) {
+                        setPB(pb3, value = i_obj, title = title_progress, label = "extracting features information")
+                        fv = readBin(toread2, "raw", n = feat_number * 4, endian = r_endian)
+                        # extract object features values at expected position
+                        if((i_obj + obj_shift) %in% objects_1[1]) {
+                          assign("objects_1", value = objects_1[-1], envir = cur_env)
+                          return(fv)
+                        }
+                        return(raw(0))
+                      }))}, finally = endPB(pb3))
+                  } else {
                     features = c(features, lapply(1:obj_number, FUN = function(i_obj) {
-                      setPB(pb3, value = i_obj, title = title_progress, label = "extracting features information")
                       fv = readBin(toread2, "raw", n = feat_number * 4, endian = r_endian)
                       # extract object features values at expected position
                       if((i_obj + obj_shift) %in% objects_1[1]) {
@@ -334,144 +315,102 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
                         return(fv)
                       }
                       return(raw(0))
-                    }))}, finally = endPB(pb3))
-                } else {
-                  features = c(features, lapply(1:obj_number, FUN = function(i_obj) {
-                    fv = readBin(toread2, "raw", n = feat_number * 4, endian = r_endian)
-                    # extract object features values at expected position
-                    if((i_obj + obj_shift) %in% objects_1[1]) {
-                      assign("objects_1", value = objects_1[-1], envir = cur_env)
-                      return(fv)
-                    }
-                    return(raw(0))
-                  }))
-                }
-                features = unlist(features)
-                if(length(features) != 4 * feat_number * length(objects)) {
-                  features = list()
-                  stop(f, "\nMismatch in object number")
-                }
-              }, error = function(e) {
-                stop(e$message)
-              }, finally = close(toread2))
-              obj_shift = obj_shift + obj_number
-            }
-          }, error = function(e) {
-            message(paste0(e$message, ". Features values were not exported"))
-          })
-        }
-        
-        if(length(features) != 0) {
-          # we were able to extract binary features values so we recreate tags 33080 and 33082
-          features = c(writeBin(as.double(L), raw(), size = 4, endian = r_endian), feat_number_r, features)
-          # add back binary feature offset
-          ifd = c(ifd, buildIFD(val = 0, typ = 4, tag = 33080, endianness = r_endian))
-          # recreate feature binary boolean flag
-          ifd = c(ifd, buildIFD(val = 1, typ = 4, tag = 33082, endianness = r_endian))
-          # add back binary feature that corresponds to extracted objects
-          ifd = c(ifd, buildIFD(val = features, typ = 1, tag = 33083, endianness = r_endian))
-        }
-        # change now time
-        ifd = c(ifd, buildIFD(val = format(Sys.time(), "%d-%m-%Y %H:%M:%S %p"), typ = 2, tag = 33004, endianness = r_endian))
-        # change user
-        ifd = c(ifd, buildIFD(val = "IFC package", typ = 2, tag = 33005, endianness = r_endian))
-        # modify total objects count
-        ifd = c(ifd, buildIFD(val = L, typ = 4, tag = 33018, endianness = r_endian))
-        # add pkg version tag /!\ mandatory to prevent overwriting original file
-        ifd = c(ifd, buildIFD(val = paste0(unlist(packageVersion("IFC")), collapse = "."), typ = 2, tag = 33090, endianness = r_endian))
-        # add fileName tag /!\ allow to track where exported objects are coming from
-        ifd = c(ifd, buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33091")),
-                                             fileName),
-                                           collapse = ">"),
-                              typ = 2, tag = 33091, endianness = r_endian))
-        # add checksum tag /!\ allow to track where exported objects are coming from
-        ifd = c(ifd, buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33092")),
-                                             checksumXIF(fileName)),
-                                           collapse = ">"), 
-                              typ = 2, tag = 33092, endianness = r_endian))
-      } else {
-        # register current object id in new tag to be able to track it
-        ifd = c(ifd, buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33093")),
-                                             OBJECT_ID),
-                                           collapse = ">"),
-                              typ = 2, tag = 33093, endianness = r_endian))
-        
-        # register current fil_ori in new tag to be able to track it
-        ifd = c(ifd, buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33094")),
-                                             fileName),
-                                           collapse = ">"), 
-                              typ = 2, tag = 33094, endianness = r_endian))
-        
-        # TODO if we remove mask tags it may be better to use offsets names as tracking object id
-
-        # modify object id
-        tmp = packBits(intToBits(floor(obj_id/2)),type="raw")
-        if(endianness!=r_endian) tmp = rev(tmp)
-        
-        # TODO ask amnis what to do with 33024
-        if(length(ifd[["33024"]])!=0) {
-          if(length(ifd[["33003"]])!=0) {
-            ifd[["33024"]]$min_content[9:12] <- ifd[["33003"]]$min_content[9:12]
+                    }))
+                  }
+                  features = unlist(features)
+                  if(length(features) != 4 * feat_number * length(objects)) {
+                    features = list()
+                    stop(f, "\nMismatch in object number")
+                  }
+                }, error = function(e) {
+                  stop(e$message)
+                }, finally = close(toread2))
+                obj_shift = obj_shift + obj_number
+              }
+            }, error = function(e) {
+              message(paste0(e$message, ". Features values were not exported"))
+            })
+          }
+          if(length(features) != 0) {
+            # we were able to extract binary features values so we recreate tags 33080 and 33082
+            features = c(writeBin(as.double(L), raw(), size = 4, endian = r_endian), feat_number_r, features)
+            extra = c(extra, 
+                    # add back binary feature offset
+                    buildIFD(val = 0, typ = 4, tag = 33080, endianness = r_endian),
+                    # recreate feature binary boolean flag
+                    buildIFD(val = 1, typ = 4, tag = 33082, endianness = r_endian),
+                    # add back binary feature that corresponds to extracted objects
+                    buildIFD(val = features, typ = 1, tag = 33083, endianness = r_endian)
+            )
           }
         }
-        if(length(ifd[["33003"]])!=0) ifd[["33003"]]$min_content[9:12] <- tmp
-      }
-      
-      # reorder ifd
-      ifd = ifd[order(as.integer(names(ifd)))]
-      
-      # define new offset position of current ifd
-      l_min = cumsum(sapply(ifd, FUN=function(i_tag) length(i_tag$min_content)))
-      l_add = cumsum(sapply(ifd, FUN=function(i_tag) length(i_tag$add_content)))
-      
-      # write this new offset
-      pos = pos + 4
-      tmp = packBits(intToBits(pos + l_add[length(l_add)]), type="raw")
-      if(endianness != r_endian) tmp = rev(tmp)
-      writeBin(object = tmp, con = towrite, endian = r_endian)
-      
-      # write all additional content
-      writeBin(object = unlist(lapply(ifd, FUN=function(i_tag) i_tag$add_content)), con = towrite, endian = r_endian)
-      
-      # modify number of directory entries
-      n_entries = length(ifd)
-      tmp = packBits(intToBits(n_entries),type="raw")
-      if(endianness!=r_endian) tmp = rev(tmp)
-      
-      # write modified number of entries
-      writeBin(object = tmp[1:2], con = towrite, endian = r_endian)
-      
-      # modify ifd val/offset of minimal content 
-      for(i in 1:length(ifd)) {
-        la = length(ifd[[i]]$add_content)
-        if(la==0) next
-        tmp = packBits(intToBits(pos),type="raw")
+        extra = c(extra, 
+                  # change now time
+                  buildIFD(val = format(Sys.time(), "%d-%m-%Y %H:%M:%S %p"), typ = 2, tag = 33004, endianness = r_endian),
+                  # change user
+                  buildIFD(val = "IFC package", typ = 2, tag = 33005, endianness = r_endian),
+                  # modify total objects count
+                  buildIFD(val = L, typ = 4, tag = 33018, endianness = r_endian),
+                  # add pkg version tag /!\ mandatory to prevent overwriting original file
+                  buildIFD(val = paste0(unlist(packageVersion("IFC")), collapse = "."), typ = 2, tag = 33090, endianness = r_endian),
+                  # add fileName tag /!\ allow to track where exported objects are coming from
+                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33091")),
+                                          fileName),
+                                        collapse = ">"),
+                           typ = 2, tag = 33091, endianness = r_endian),
+                  # add checksum tag /!\ allow to track where exported objects are coming from
+                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33092")),
+                                          checksumXIF(fileName)),
+                                        collapse = ">"), 
+                           typ = 2, tag = 33092, endianness = r_endian)
+        )
+      } else {
+        extra = c(extra, 
+                  # register current object id in new tag to be able to track it
+                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33093")),
+                                          OBJECT_ID),
+                                        collapse = ">"),
+                           typ = 2, tag = 33093, endianness = r_endian),
+                  
+                  # register current fil_ori in new tag to be able to track it
+                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33094")),
+                                          fileName),
+                                        collapse = ">"), 
+                           typ = 2, tag = 33094, endianness = r_endian)
+        )
+        
+        # TODO if we remove mask tags it may be better to use offsets names as tracking object id
+        # modify object id
+        tmp = cpp_uint32_to_raw(floor(obj_id/XIF_step))
         if(endianness!=r_endian) tmp = rev(tmp)
-        ifd[[i]]$min_content[9:12] <- tmp
-        pos = pos + la
+        if(length(ifd[["33003"]])!=0) ifd[["33003"]]$raw[9:12] <- tmp
+        
+        # TODO ask amnis what to do with 33024
+        # if(length(ifd[["33024"]])!=0) {
+        #   if(length(ifd[["33003"]])!=0) {
+        #     ifd[["33024"]]$min_content[9:12] <- ifd[["33003"]]$min_content[9:12]
+        #   }
+        # }
+        # if(length(ifd[["33003"]])!=0) {
+        #   if(length(ifd[["33024"]])!=0) {
+        #     ifd[["33024"]]$min_content[9:12] <- ifd[["33003"]]$min_content[9:12]
+        #   } else {
+        #     ifd = c(ifd, buildIFD(val = OBJECT_ID, typ = 4, tag = 33024, endianness = r_endian))
+        #   }
+        #   ifd[["33003"]]$min_content[9:12] <- tmp
+        # }
       }
       
-      # modify 33080 feature offset to point to features in 33083
-      if("33080" %in% names(ifd)) {
-        tmp = packBits(intToBits(l_add["33083"] - length(ifd[["33083"]]$add_content) + 8),type="raw")
-        if(endianness!=r_endian) tmp = rev(tmp)
-        ifd[["33080"]]$min_content[9:12] <- tmp
-      }
-      
-      # write ifd
-      writeBin(object = unlist(lapply(ifd, FUN=function(i_tag) i_tag$min_content)), con = towrite, endian = r_endian)
-      pos = pos + l_min[length(l_min)] + 2
+      pos = writeIFD(ifd, r_con = toread, w_con = towrite, pos = pos, extra = extra, endianness = r_endian)
       obj_id = obj_id + 1
     }
-    writeBin(object = packBits(intToBits(0),type="raw"), con = towrite, endian = r_endian)
+    writeBin(object = cpp_uint32_to_raw(0), con = towrite, endian = r_endian)
   }, error = function(e) {
-    close(towrite)
     stop(paste0("Can't create 'write_to' file.\n", write_to,
                 ifelse(overwritten,"\nFile was not modified.\n","\n"),
                 "See pre-file @\n", normalizePath(file_w, winslash = "/", mustWork = FALSE), "\n",
                 e$message), call. = FALSE)
-  })
-  close(towrite)
+  }, finally = close(towrite))
   if(overwritten) {
     mess = paste0("\n######################\n", write_to, "\nhas been successfully overwritten\n")
     if(!suppressWarnings(file.rename(to = write_to, from = file_w))) { # try file renaming which is faster
