@@ -6,7 +6,7 @@
   -IFC: An R Package for Imaging Flow Cytometry                                 
   -YEAR: 2020                                                                   
   -COPYRIGHT HOLDERS: Yohann Demont, Gautier Stoll, Guido Kroemer,              
-                      Jean-Pierre Marolleau, Loï?c Gaççon,                       
+                      Jean-Pierre Marolleau, Loïc Garçon,                       
                       INSERM, UPD, CHU Amiens                                   
  
  
@@ -40,6 +40,7 @@
 #include "tiff.hpp"
 #include "resize.hpp"
 #include "matrix_logic.hpp"
+#include "align.hpp"
 using namespace Rcpp;
 
 //' @title Matrix Normalization
@@ -181,13 +182,15 @@ Rcpp::NumericMatrix hpp_mark (const Rcpp::NumericMatrix A,
                               const bool invert = false) {
   R_len_t bc = B.ncol();
   R_len_t br = B.nrow();
+  R_len_t xxoff = xoff < 0 ? 0 : xoff;
+  R_len_t yyoff = yoff < 0 ? 0 : yoff;
   if((A.ncol() < (bc + xoff)) || (A.nrow() < (br + yoff))) Rcpp::stop("hpp_mark: A should be at least of same dimensions as 'B' + 'offsets'");
   if((mask.ncol() < bc) || (mask.nrow() < br)) Rcpp::stop("hpp_mark: 'mask' should be at least of same dimensions as 'B'");
   Rcpp::NumericMatrix out = Rcpp::clone(A);
   if(invert) {
-    for(R_len_t y = 0; y < br; y++) for(R_len_t x = 0; x < bc; x++) if(mask(y,x)) out(y+yoff,x+xoff) = std::fabs(1-B(y,x));
+    for(R_len_t y = 0; y < br; y++) for(R_len_t x = 0; x < bc; x++) if(mask(y,x)) out(y+yyoff,x+xxoff) = std::fabs(1-B(y,x));
   } else {
-    for(R_len_t y = 0; y < br; y++) for(R_len_t x = 0; x < bc; x++) if(mask(y,x)) out(y+yoff,x+xoff) = B(y,x);
+    for(R_len_t y = 0; y < br; y++) for(R_len_t x = 0; x < bc; x++) if(mask(y,x)) out(y+yyoff,x+xxoff) = B(y,x);
   }
   return out;
 }
@@ -212,7 +215,9 @@ Rcpp::NumericMatrix hpp_mark (const Rcpp::NumericMatrix A,
 //' @param full_range bool, only apply when mode is not "raw", if full_range is TRUE, then 'input_range' will be set to 'c(0,4095)' and gamma forced to 1. Default is false.
 //' @param force_range bool, only apply when mode is not "raw", if force_range is TRUE, then 'input_range' will be adjusted to mat range in [-4095, +inf] and gamma forced to 1. Default is false.\cr
 //' Note that this parameter takes the precedence over 'input_range' and 'full_range'.
-//' @param gamma correction. Default is 1, for no correction.
+//' @param gamma correction. Default is 1.0, for no correction.
+//' @param spatialX X offset correction. Default is 0.0 for no change.
+//' @param spatialY Y offset correction. Default is 0.0 for no change.
 //' @details When add_noise is false, background will be automatically set to minimal pixel value for "masked" and "MC" removal method.\cr
 //' when a mask is detected, add_noise, full_range and force_range are set to false, background mean and sd to 0, and input_range to [0,3].\cr
 //' @keywords internal
@@ -230,7 +235,9 @@ Rcpp::NumericVector hpp_transform(const Rcpp::NumericMatrix mat,
                                   const double sd = 0.0,
                                   const bool full_range = false,
                                   const bool force_range = false,
-                                  const double gamma = 1.0) {
+                                  const double gamma = 1.0,
+                                  const double spatialX = 0.0,
+                                  const double spatialY = 0.0) {
   Rcpp::NumericMatrix foo;
   Rcpp::CharacterVector removal = Rcpp::wrap(msk.attr("removal"));
   Rcpp::CharacterVector col_name = Rcpp::wrap(color.attr("names"));
@@ -238,12 +245,12 @@ Rcpp::NumericVector hpp_transform(const Rcpp::NumericMatrix mat,
   if(type == 3) { // a mask is detected parameters are forced
     Rcpp::NumericVector msk_range = Rcpp::NumericVector::create(0.0, 3.0);
     if((removal[0] == "none") || (removal[0] == "raw")) {
-      foo = Rcpp::clone(mat);
-      foo.attr("mask") = msk;
+      foo = hpp_align_img(mat, spatialX, spatialY);
+      foo.attr("mask") = hpp_align_msk(msk, spatialX, spatialY);
     } else {
-      foo = hpp_cleanse(mat, msk, false, 0.0, 0.0);
+      foo = hpp_cleanse(hpp_align_img(mat, spatialX, spatialY), hpp_align_msk(msk, spatialX, spatialY), false, 0.0, 0.0);
     } 
-    foo = hpp_resize(foo, size[0], size[1], false, 0.0, 0.0);
+    foo = hpp_resize(foo, false, 0.0, 0.0);
     if(mode != "raw") {
       foo = hpp_normalize(foo, msk_range, false, false, 1.0);
     }
@@ -285,10 +292,10 @@ Rcpp::NumericVector hpp_transform(const Rcpp::NumericMatrix mat,
       sd_2 = sd;
     }
     if((removal[0] == "none") || (removal[0] == "raw")) {
-      foo = Rcpp::clone(mat);
-      foo.attr("mask") = msk;
+      foo = hpp_align_img(mat, spatialX, spatialY);
+      foo.attr("mask") = hpp_align_msk(msk, spatialX, spatialY);
     } else {
-      foo = hpp_cleanse(mat, msk, add_noise, bg_2, sd_2);
+      foo = hpp_cleanse(hpp_align_img(mat, spatialX, spatialY), hpp_align_msk(msk, spatialX, spatialY), add_noise, bg_2, sd_2);
     } 
     foo = hpp_resize(foo, size[0], size[1], add_noise, bg_2, sd_2);
     if(mode != "raw") {
@@ -336,6 +343,8 @@ Rcpp::NumericVector hpp_transform(const Rcpp::NumericMatrix mat,
 //' @param physicalChannel CharacterVector of indices for each channels
 //' @param xmin NumericVector of minimal values for each channels
 //' @param xmax NumericVector of maximal values for each channels
+//' @param spatialX NumericVector of X spatial offset correction for each channels
+//' @param spatialY NumericVector of Y spatial offset correction for each channels 
 //' @param removal IntegerVector of removal method to be used for each channels
 //' @param add_noise LogicalVector of whether to add_noise for each channels
 //' @param full_range LogicalVector of whether to use full_range for each channels
@@ -349,7 +358,7 @@ Rcpp::NumericVector hpp_transform(const Rcpp::NumericMatrix mat,
 //' - 3: at least one masked\cr
 //' - 4: at least one MC
 //' @param mode string, color mode export. Either "rgb", "gray" or "raw". Default is "raw".
-//' @param size a length 2 IntegerVector of final dimensions (height,width) of the image. Default is 0,0 for no change.\cr
+//' @param size a length 2 IntegerVector of final dimensions (height,width) of the image. Default is 0,0 for no change.
 //' @param verbose bool, whether to display information (use for debugging purpose). Default is false.
 //' @keywords internal
 ////' @export
@@ -360,6 +369,8 @@ Rcpp::List hpp_extract (const std::string fname,
                         const Rcpp::CharacterVector physicalChannel,
                         const Rcpp::NumericVector xmin,
                         const Rcpp::NumericVector xmax,
+                        const Rcpp::NumericVector spatialX,
+                        const Rcpp::NumericVector spatialY,
                         const Rcpp::IntegerVector removal,
                         const Rcpp::LogicalVector add_noise,
                         const Rcpp::LogicalVector full_range,
@@ -391,8 +402,16 @@ Rcpp::List hpp_extract (const std::string fname,
     break;
   }
   case 2: { // an image is detected use background mean and sd
-    bg_2 = Rcpp::clone(Rcpp::as<Rcpp::NumericVector>(infos["BG_MEAN"]));
-    sd_2 = Rcpp::clone(Rcpp::as<Rcpp::NumericVector>(infos["BG_STD"]));
+    if(nNotisNULL(infos["BG_MEAN"])) {
+      bg_2 = Rcpp::clone(Rcpp::as<Rcpp::NumericVector>(infos["BG_MEAN"]));
+    } else {
+      bg_2 = Rcpp::rep(0.0, nb_channels);
+    }
+    if(nNotisNULL(infos["BG_STD"])) {
+      sd_2 = Rcpp::clone(Rcpp::as<Rcpp::NumericVector>(infos["BG_STD"]));
+    } else {
+      sd_2 = Rcpp::rep(0.0, nb_channels);
+    }
     break;
   }
   case 3: { // a mask is detected some parameters are forced
@@ -410,7 +429,7 @@ Rcpp::List hpp_extract (const std::string fname,
                         imw, iml, nb_channels,
                         0, com, verbose);
   Rcpp::IntegerMatrix msk_init(iml, imw / nb_channels);
-  msk_init.fill(0);
+  // msk_init.fill(0);
   Rcpp::IntegerMatrix msk = Rcpp::clone(msk_init);
   Rcpp::IntegerMatrix MC = Rcpp::clone(msk_init);
   
@@ -475,6 +494,7 @@ Rcpp::List hpp_extract (const std::string fname,
         break;
       }
       }
+      // if(masks.hasAttribute("status")) msk.attr("removal") = "invalid";
       
       out[i] = hpp_transform(img[chan_idx],
                              colors[chan_idx],
@@ -488,7 +508,9 @@ Rcpp::List hpp_extract (const std::string fname,
                                                 sd_2[chan_idx],
                                                     full_range[chan_idx],
                                                               force_range[chan_idx],
-                                                                         gamma[chan_idx]);
+                                                                         gamma[chan_idx],
+                                                                              spatialX[chan_idx],
+                                                                                      spatialY[chan_idx]);
     }
   } else {
     msk.attr("removal") = "none";
@@ -507,7 +529,9 @@ Rcpp::List hpp_extract (const std::string fname,
                                                 sd_2[chan_idx],
                                                     full_range[chan_idx],
                                                               force_range[chan_idx],
-                                                                         gamma[chan_idx]);
+                                                                         gamma[chan_idx],
+                                                                              spatialX[chan_idx],
+                                                                                      spatialY[chan_idx]);
     }
   }
   return out;
